import os
import threading
from typing import Any, Type, TypeVar, Optional, cast

T = TypeVar("T", bound="Config")


class Config(object):
    _instance: Optional["Config"] = None
    _lock = threading.Lock()

    sandbox_manager_url: str
    agent_planner_url: str
    auth_token: str

    def __new__(cls: Type[T], *args: Any, **kwargs: Any) -> T:
        if cls._instance is None:
            with cls._lock:
                if cls._instance is None:
                    instance = super().__new__(cls)
                    cls._instance = instance
        return cast(T, cls._instance)

    def __init__(
        self,
            agent_planner_url: str = "",
            sandbox_manager_url: str = "",
            auth_token: str = "",
    ) -> None:
        if hasattr(self, "initialized") and self.initialized:
            return

        self.agent_planner_url = agent_planner_url or os.environ.get("AGENT_PLANNER_URL")
        self.sandbox_manager_url = sandbox_manager_url or os.environ.get("SANDBOX_MANAGER_URL")
        self.auth_token = auth_token or os.environ.get("AUTH_TOKEN")
        self.initialized = True

    def __repr__(self) -> str:
        return (
            f"sandbox_manager_url={self.sandbox_manager_url!r} "
            f"agent_planner_url={self.auth_token!r} "
            f"auth_token={self.auth_token!r}"
        )


config = Config()


def get_config() -> Config:
    return config


def set_config(
    agent_planner_url: Optional[str] = "",
    sandbox_manager_url: Optional[str] = "",
    auth_token: Optional[str] = "",
) -> None:
    config.agent_planner_url = agent_planner_url or config.agent_planner_url
    config.sandbox_manager_url = sandbox_manager_url or config.sandbox_manager_url
    config.auth_token = auth_token or config.auth_token
