from typing import List, Optional, Union, Dict, Any, Literal, Tuple
import logging

from .models.common_models import (
    Action,
    ScreenshotResult,
    ComputerActionArgs,
    SandboxDetails,
    SandboxOsType
)
from .models.tool_server_models import BaseResponse
from .services.sandbox_manager import SandboxManagerClient
from .services.sandbox_tool_server import AsyncToolServerClient
from .config import get_config

logger = logging.getLogger(__name__)


class Sandbox:
    def __init__(self, details: SandboxDetails, auth_token: str=None, sandbox_manager_url: str=None):
        self.details = details
        _config = get_config()
        self.auth_token = auth_token or _config.auth_token
        self.sandbox_manager_url = sandbox_manager_url or _config.sandbox_manager_url
        if not self.auth_token:
            raise Exception("Conf: auth token is not provided")
        if not self.sandbox_manager_url:
            raise Exception("Conf: ecs manager url is not provided")
        self.sandbox_manager = SandboxManagerClient(endpoint=self.sandbox_manager_url, auth_token=self.auth_token)

        if details.tool_server_endpoint:
            tool_server_endpoint = details.tool_server_endpoint
        elif details.tool_server_proxy_uri:
            tool_server_endpoint = self.sandbox_manager_url + details.tool_server_proxy_uri
        else:
            logger.warning(f"Sandbox {details.id} initialized without a tool server endpoint. "
                           f"The CUA application may used a serverless gateway.")
            raise Exception("Get no tool server endpoint from sandbox detail info")

        self.tool_server_client = AsyncToolServerClient(tool_server_endpoint, auth_key=self.auth_token,
                                                        sandbox_id=self.details.id)

    @property
    def id(self) -> str:
        return self.details.id

    @property
    def ip_address(self) -> Optional[str]:
        return self.details.primary_ip

    @property
    def os_type(self) -> Optional[str]:
        return self.details.os_type

    @property
    def tool_server_endpoint(self) -> Optional[str]:
        return self.tool_server_client.base_url

    async def delete(self) -> Dict[str, Any]:
        """删除此实例。"""
        return await self.sandbox_manager.delete_sandbox(self.id)

    async def screenshot(self) -> ScreenshotResult:
        """获取屏幕截图。"""
        screenshot_response = await self.tool_server_client.take_screenshot()
        screen_size_response = await self.tool_server_client.get_screen_size()
        if not screen_size_response:
            raise Exception("Get screen shot failed")
    
        result = ScreenshotResult(
            width=screen_size_response.Result.width,
            height=screen_size_response.Result.height,
            base_64_image=screenshot_response.Result.screenshot
        )
        return result

    async def get_stream_url(self) -> str:
        """获取用于监控或交互的实例流式URL (例如 VNC/RDP over WebSocket or custom stream)。"""
        if not self.id:
            raise ValueError("Sandbox ID is not available to construct stream URL.")
        if not self.ip_address:
            raise ValueError("Sandbox IP address is not available to construct stream URL.")
        
        response = await self.sandbox_manager.describe_sandbox_terminal_url(self.id)
        stream_url = response.get('Result', {}).get('Url')
        if not stream_url:
            if self.os_type == SandboxOsType.WINDOWS.value:
                logger.warning("Get empty stream url from ecs manager. You may not have filled in AK/SK when creating the CUA application.")
            elif self.os_type == SandboxOsType.LINUX.value:
                logger.warning("Get empty stream url from ecs manager. In the case of non-standard gateways, "
                               "linux sandboxes currently do not support obtaining remote stream url in .")
        return stream_url

    async def computer(
        self,
        action: Action,
        coordinates: Optional[List[int]] = None,
        hold_keys: Optional[List[str]] = None,
        text: Optional[str] = None,
        button: Optional[str] = None,
        num_clicks: Optional[int] = 1,
        path: Optional[List[List[int]]] = None,
        delta_x: Optional[int] = None,
        delta_y: Optional[int] = None,
        keys: Optional[List[str]] = None,
        duration: Optional[Union[int, float]] = None,
        screenshot: bool = True,
        press: bool = False,
        release: bool = False,
        scroll_direction: Optional[Literal["up", "down", "left", "right"]] = None,
        scroll_amount: Optional[int] = None
    ) -> Optional[BaseResponse]:
        """执行计算机操作。"""
        ts_client = self.tool_server_client
        args = ComputerActionArgs(
            action=action,
            coordinates=coordinates,
            hold_keys=hold_keys,
            text=text,
            button=button,
            num_clicks=num_clicks,
            path=path,
            delta_x=delta_x,
            delta_y=delta_y,
            keys=keys,
            duration=duration,
            screenshot=screenshot,
            press=press,
            release=release,
            scroll_direction=scroll_direction,
            scroll_amount=scroll_amount
        )
        return await ts_client.computer_action(args)

    def __repr__(self) -> str:
        return f"<Sandbox id='{self.id}' type='{self.details.os_type}' status='{self.details.status}' endpoint='{self.tool_server_endpoint}'>"